
;TV's server version 3.00
;Copyright (c) 2014, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;The GetCopyOfInternalProtocolStructures function will create a copy of the internal protocol structures and returns the first structure in the linked list.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    GetCopyOfInternalProtocolStructures_c:
    StartProc 1

    mov rcx,retGetCopyOfInternalProtocolStructures_c
    xchg qword [rsp],rcx
    jmp GetCopyOfInternalProtocolStructures_asm

    retGetCopyOfInternalProtocolStructures_c:
    sub rsp,psize
    EndProc 1
    jmp rcx
else
    GetCopyOfInternalProtocolStructures_c:
end if

;---------
;asm implementation
;---------
GetCopyOfInternalProtocolStructures_asm:
;get internal protocol structure
push pdword [pesp+psize] ;psess
call getInternalProtocolStructure
test peax,peax
jz endGetCopyOfInternalProtocolStructures

  mov eax,dword [peax+(psize*7)+4] ;get member type
  and eax,PT_CONTROLPANEL ;check on CONTROL_PANEL rights
  jz endGetCopyOfInternalProtocolStructures ;jump if no control panel rights

mov peax,[protocols]	;return pointer to first internal protocol structure

endGetCopyOfInternalProtocolStructures:
ret psize

;------------------------------------------------------------------------------
;The ReleaseCopyOfInternalProtocolStructures function will release all recources allocated by function GetCopyOfInternalProtocolStructures.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    ReleaseCopyOfInternalProtocolStructures_c:
    StartProc 1

    mov rcx,retReleaseCopyOfInternalProtocolStructures_c
    xchg qword [rsp],rcx
    jmp ReleaseCopyOfInternalProtocolStructures_asm

    retReleaseCopyOfInternalProtocolStructures_c:
    sub rsp,psize*1
    EndProc 1
    jmp rcx
else
    ReleaseCopyOfInternalProtocolStructures_c:
end if

;---------
;asm implementation
;---------
ReleaseCopyOfInternalProtocolStructures_asm:
;call denter

;since GetCopyOfInternalProtocolStructures does not make a copy, but return internal pointer, do nothing...

endReleaseCopyOfInternalProtocolStructures:
;call dleave
ret psize*1

;------------------------------------------------------------------------------
;The CountListenSessions function will return the ammount of active ListenSessions.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    CountListenSessions_c:
    StartProc 1

    mov rcx,retCountListenSessions_c
    xchg qword [rsp],rcx
    jmp CountListenSessions_asm

    retCountListenSessions_c:
    sub rsp,psize*1
    EndProc 1
    jmp rcx
else
    CountListenSessions_c:
end if

;---------
;asm implementation
;---------
CountListenSessions_asm:
;get internal protocol structure
push pdword [pesp+psize] ;psess
call getInternalProtocolStructure
test peax,peax
jz endCountListenSessions

  mov eax,dword [peax+(psize*7)+4] ;get member type
  and eax,PT_CONTROLPANEL ;check on CONTROL_PANEL rights
  jz endCountListenSessions ;jump if no control panel rights


;get the ammount of items in the collection listenSessions
push [listenSessions]
call CountItems
shr eax,1 ;EAX = EAX / 2. Divide by two because the items are indexed by the Listensession (lsess) and we shouldn't count those.

endCountListenSessions:
ret psize

;------------------------------------------------------------------------------
;The GetCopyOfInternalListenStructure function will create a copy of the internal Listen structure and returns the ListenSession.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    GetCopyOfInternalListenStructure_c:
    StartProc 3

    mov rcx,retGetCopyOfInternalListenStructure_c
    xchg qword [rsp],rcx
    jmp GetCopyOfInternalListenStructure_asm

    retGetCopyOfInternalListenStructure_c:
    sub rsp,psize*3
    EndProc 3
    jmp rcx
else
    GetCopyOfInternalListenStructure_c:
end if

;---------
;asm implementation
;---------
GetCopyOfInternalListenStructure_asm:
call denter

;get internal protocol structure
push pdword [ebp+psize*2] ;psess
call getInternalProtocolStructure
test peax,peax
jz endGetCopyOfInternalListenStructure

  mov eax,dword [peax+(psize*7)+4] ;get member type
  and eax,PT_CONTROLPANEL ;check on CONTROL_PANEL rights
  jz endGetCopyOfInternalListenStructure ;jump if no control panel rights

;lock the list for write actions
push [listenSessions]
call LockListWriteAcces

mov ecx,dword [pebp+psize*3] ;parameter id
shl ecx,1 ;ECX = ECX * 2 (since the list is indexed)
dec ecx   ;--ECX since we want the index (this is a pointer to the ListenSession)

;get the ListenSession
push pecx
push [listenSessions]
call GetDwordItem
test peax,peax
jz endAndUnlockGetCopyOfInternalListenStructure

mov pesi,pdword [pebp+psize*4] ;paramter *lsess
test pesi,pesi
jz dontReturnListenSession

  mov pdword [pesi],peax ;set *lsess parameter

dontReturnListenSession:
inc ecx ;++ECX; since we want the the InternalListenSessionStructure.

;get the InternalListenSession
push pecx
push [listenSessions]
call GetDwordItem
test peax,peax
jz endAndUnlockGetCopyOfInternalListenStructure

mov pesi,peax ;(R/E)SI => InternalListenSession Structure

;create and InternalListenSession Structure
push SIZEOF_INTERNALLISTENSESSION
call GetMemLocation
mov pebx,peax ;(R/E)BX => new created InternalListenSession Structure

if used is32
  mov eax,dword [pesi+psize*5] ;EAX = member status from original InternalListenSession Structure
  mov dword [pebx+psize*5],eax ;Set member status in the new created InternalListenSession Structure

  mov eax,dword [pesi+(psize*5)+4] ;EAX = member id from original InternalListenSession Structure
  mov dword [pebx+(psize*5)+4],eax ;Set member id in the new created InternalListenSession Structure
else
  mov peax,qword [pesi+psize*5] ;EAX = member status and id from original InternalListenSession Structure
  mov qword [pebx+psize*5],peax ;Set member status and id in the new created InternalListenSession Structure
end if

;copy the name
push pdword [pesi+(5*psize)+8+psize]
call copyStringA
mov pdword [pebx+(5*psize)+8+psize],peax ;fill in the ListenSession name

;Create an collection pointer without acces flags (so a direct pointer) and size as pointer (not integer) so it is aligned for C compatibility
mov pesi,pdword [pesi] ;get pointer to bindedToHandle

;make a copy of the collection
push pesi
call copyDwordList
mov pesi,pdword [peax] ;get pionter to the actual collection
mov pdword [pebx],pesi ;set member BindedToHandles in new created InternalListenSession

;free the 'header' of the collection
push peax
call FreeMemLocation

mov peax,pebx ;Set the new created InternalListenSession Structure as return value

endAndUnlockGetCopyOfInternalListenStructure:
push peax

;unlock the list for write actions
push [listenSessions]
call ExternalUnlockListWriteAcces

pop peax

endGetCopyOfInternalListenStructure:
call dleave
ret psize*3

;---------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;The GetCopyOfInternalListenStructureByLsess function will create a copy of the internal Listen structure that belongs to the passed ListenSession.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    GetCopyOfInternalListenStructureByLsess_c:
    StartProc 2

    mov rcx,retGetCopyOfInternalListenStructureByLsess_c
    xchg qword [rsp],rcx
    jmp GetCopyOfInternalListenStructureByLsess_asm

    retGetCopyOfInternalListenStructureByLsess_c:
    sub rsp,psize*2
    EndProc 2
    jmp rcx

else
    GetCopyOfInternalListenStructureByLsess_c:
end if

;---------
;asm implementation
;---------
GetCopyOfInternalListenStructureByLsess_asm:
call denter

;lock the list for write actions
push [listenSessions]
call LockListWriteAcces

;get Id for the ListenSession
push pdword [pebp+psize*3]  ;lsess to look for
push [listenSessions]	    ;collection
call FindDword
test eax,eax
jz endGetCopyOfInternalListenStructureByLsess

inc eax
shr eax,1 ;divide by 2 to the the Id

push 0	  ;output ListenSession
push peax ;id
push pdword [pebp+psize*2]  ;psess
call GetCopyOfInternalListenStructure_asm

push peax

;unlock the list for write actions
push [listenSessions]
call ExternalUnlockListWriteAcces

pop peax

endGetCopyOfInternalListenStructureByLsess:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The ReleaseCopyOfInternalListenStructure function will release all recources allocated by function GetCopyOfInternalListenStructure.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    ReleaseCopyOfInternalListenStructure_c:
    StartProc 1

    mov rcx,retReleaseCopyOfInternalListenStructure_c
    xchg qword [rsp],rcx
    jmp ReleaseCopyOfInternalListenStructure_asm

    retReleaseCopyOfInternalListenStructure_c:
    sub rsp,psize*1
    EndProc 1
    jmp rcx
else
    ReleaseCopyOfInternalListenStructure_c:
end if

;---------
;asm implementation
;---------
ReleaseCopyOfInternalListenStructure_asm:

mov peax,pdword [pesp+psize] ;parameter cpyInternalListenStructure

push pdword [peax] ;push for FreeMemLocation (to free BindedToHandles)
push pdword [peax+(5*psize)+8+psize] ;push name
push peax	   ;push for FreeMemLocation (to free InternalListenStructure)

call FreeMemLocation
call FreeMemLocation
call FreeMemLocation

endReleaseCopyOfInternalListenStructure:
ret psize*1

;------------------------------------------------------------------------------
;The RegisterProtocolLoadedCallbackFunction function will register a callback function that will be called every time a new protocol is loaded.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RegisterProtocolLoadedCallbackFunction_c:
    StartProc 2

    mov rcx,retRegisterProtocolLoadedCallbackFunction_c
    xchg qword [rsp],rcx
    jmp RegisterProtocolLoadedCallbackFunction_asm

    retRegisterProtocolLoadedCallbackFunction_c:
    sub rsp,psize*2
    EndProc 2
    jmp rcx
else
    RegisterProtocolLoadedCallbackFunction_c:
end if

;---------
;asm implementation
;---------
RegisterProtocolLoadedCallbackFunction_asm:
call denter

;get internal protocol structure
push pdword [ebp+psize*2] ;psess
call getInternalProtocolStructure
test peax,peax
jz endRegisterProtocolLoadedCallbackFunction

  mov eax,dword [peax+(psize*7)+4] ;get member type
  and eax,PT_CONTROLPANEL ;check on CONTROL_PANEL rights
  jz endRegisterProtocolLoadedCallbackFunction ;jump if no control panel rights

push pdword [pebp+psize*3] ;function to register
push [registerdProtocolIsLoadedFunctions]
call AddDword

xor eax,eax
inc eax ;return true

endRegisterProtocolLoadedCallbackFunction:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;The RegisterProtocolLoadedCallbackFunction function will register a callback function that will be called every time a new protocol is loaded.
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    RegisterListenSessionIsCreatedCallbackFunction_c:
    StartProc 2

    mov rcx,retRegisterListenSessionIsCreatedCallbackFunction_c
    xchg qword [rsp],rcx
    jmp RegisterListenSessionIsCreatedCallbackFunction_asm

    retRegisterListenSessionIsCreatedCallbackFunction_c:
    sub rsp,psize*2
    EndProc 2
    jmp rcx
else
    RegisterListenSessionIsCreatedCallbackFunction_c:
end if

;---------
;asm implementation
;---------
RegisterListenSessionIsCreatedCallbackFunction_asm:
call denter

;get internal protocol structure
push pdword [ebp+psize*2] ;psess
call getInternalProtocolStructure
test peax,peax
jz endRegisterListenSessionIsCreatedCallbackFunction

  mov eax,dword [peax+(psize*7)+4] ;get member type
  and eax,PT_CONTROLPANEL ;check on CONTROL_PANEL rights
  jz endRegisterListenSessionIsCreatedCallbackFunction ;jump if no control panel rights

push pdword [pebp+psize*3] ;function to register
push [registerdListenSessionIsCreatedFunctions]
call AddDword

xor eax,eax
inc eax ;return true

endRegisterListenSessionIsCreatedCallbackFunction:
call dleave
ret psize*2