
;TV's server version 3.00
;Copyright (c) 2014, Tycho Veltmeijer.
;All rights reserved.

;This include file conains the functions:
;SetProtocolInformation (C and asm)
;SetControlPanelOptions (C and asm)
;AcceptProtocol (C and asm)
;loadPlugin: This function will load the protocols of the given modules
;__asm_get_export_table:  This function will send the export table of this program  to a assembly based plugin
;__asm_set_export_table:  This function will recieve the export table of the plugin that is loading by function loadPlugin

;------------------------------------------------------------------------------
;This function will set hte protocolname in the blocklist, at next restart, the protocol won't be able to load
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    CloseProtocolSession_c:
    StartProc 1

    mov rcx,retCloseProtocolSession_c
    xchg qword [rsp],rcx
    jmp CloseProtocolSession_asm

    retCloseProtocolSession_c:
    ;mov qword [rsp],rcx
    sub rsp,1*psize
    EndProc 1
    ;ret
    jmp rcx
else
    CloseProtocolSession_c:
end if

;---------
;asm implementation
;---------

CloseProtocolSession_asm:
call denter

mov pecx,endCloseProtocolSession
call checkProtocolSession

push peax
call SaveBlockedPluginToDisk

endCloseProtocolSession:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will set the information of the protocol
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    SetProtocolInformation_c:
    StartProc 5

    mov rcx,retSetProtocolInformation_c
    xchg qword [rsp],rcx
    jmp SetProtocolInformation_asm

    retSetProtocolInformation_c:
    ;mov qword [rsp],rcx
    sub rsp,5*psize
    EndProc 5
    ;ret
    jmp rcx
else
    SetProtocolInformation_c:
end if

;---------
;asm implementation
;---------

SetProtocolInformation_asm:
call denter

mov pecx,returnSetProtocolInformation
call checkProtocolSession


;move internal protocol structure to ebx/rbx
mov pebx,peax

;get type from stack
mov peax,pdword [pebp+(psize*3)] ;type
mov edx,eax

;check if type > PT_LOWEST_FLAG, then return error
cmp ax,PT_HIGHEST_FLAG
jng type_is_ok

mov eax,PT_WRONG_TYPE
jmp endSetProtocolInformation

type_is_ok:
;check version
mov eax,dword [pebp+(psize*6)]
test eax,eax
jz versionIsSupported

mov eax,VERSION_UNSUPPORTED
jmp endSetProtocolInformation

versionIsSupported:
; mov type, extraSize members in InternalProtocolStructure
mov dword [pebx+(7*psize)+4],edx     ;type
mov dword [pebx+(7*psize)+(4*2)],eax ;version

;fill in extra size parameter
mov peax,pdword [pebp+(psize*5)]
mov dword [pebx+(7*psize)+(4*3)],eax ;extraSize

;fill in the pSign member
push pdword [pebp+(psize*4)]
call copyProtocolSignature
mov pdword [pebx+POS_PSIGN_INTERNALPROTOCOL],peax ;pSign

;fill in status
mov dword [pebx+(psize*7)],PT_STARTING

xor eax,eax ;NO_ERROR

;--
endSetProtocolInformation:
call setLastError

returnSetProtocolInformation:
call dleave
ret (5*psize)

;------------------------------------------------------------------------------
;This function will set the control panel options for the protocol
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    SetControlPanelOptions_c:
    StartProc 2

    mov rcx,retSetControlPanelOptions_c
    xchg qword [rsp],rcx
    jmp SetControlPanelOptions_asm

    retSetControlPanelOptions_c:
    ;mov qword [rsp],rcx
    sub rsp,2*psize
    ;ret
    EndProc 2
    jmp rcx
else
    SetControlPanelOptions_c:
end if

;---------
;asm implementation
;---------

SetControlPanelOptions_asm:
call denter

mov pecx,returnSetControlPanelOptions
call checkProtocolSession
;move internal protocol structure to edi/rdi
mov pedi,peax



;check pointer to ControlPanelOptions structure
push endSetControlPanelOptions
push RIGHT_READ
push pdword [pebp+(psize*3)]
call saveCheckPointer

;get member main of ControlPanelOptions
mov pebx,pdword [pebp+psize*3]
mov ecx,dword [pebx] ;ecx = size of structure

;get pointer to 'main'  member
sub ecx,8

;if the structure is empty, don't do annything
jz returnSetControlPanelOptions

add pebx,8

;the main structure size is greater than the size of the total strucure... return
;check is size<0 then size=SIZEOF_CONTROLPANELOPTION
mov eax,dword [pebx]
test eax,eax
jnz compairSizeMain

mov eax,SIZEOF_CONTROLPANELOPTION

compairSizeMain:
cmp ecx,eax
jb returnSetControlPanelOptions

;make a copy of the main member
push pebx
call copyControlPanelOption
mov pdword [pedi+5*psize],peax


;check is size<0 then size=SIZEOF_CONTROLPANELOPTION
mov eax,dword [pebx]
test eax,eax
jnz compairSizeMain2

mov eax,SIZEOF_CONTROLPANELOPTION

compairSizeMain2:
;substract the size of the main member
sub ecx,eax
;get subOptions
add pebx,peax

test ecx,ecx
jz returnSetControlPanelOptions

;is this structure size greater than the size of the total strucure... return
cmp ecx,dword [pebx]
jb returnSetControlPanelOptions


loopSubOptionsStructure:
  push pebx
  call copyControlPanelOption

  push peax  ;save new ControlPanelOption

  ;--test dword [peax+psize+4],CPO_GEN_PROTOCOL_OPTION        do not check this
  ;--jz NoUserValueOption ;this list is not for the protocolSession

  mov peax,pdword [peax+psize+8+(7*psize)] ;get pointer to UserValueOption
  test peax,peax
  jz NoUserValueOption ;no pointer to UserValueOption

  cmp dword [peax],TD_LIST
  jne NoUserValueOption ;jump if not a list

  mov peax,pdword [peax+8] ;defaultValue
  test peax,peax
  jz NoUserValueOption

  if used is64
    mov edx,dword [peax]  ;size member
    add peax,pedx	  ;go to the end of the structure
  else
    add peax,dword [peax] ;go to the end of the structure by incrmenting by the size member
  end if
  sub peax,psize	  ;go to member 'rows'

  pop pedx  ;pop the pointer to ControlPanelOption
  push pedx ;push the pointer to ControlPanelOption
  mov edx,dword [pedx+psize+4] ;get the flags parameter
  and edx,GDPD_NAME_IS_INTEGER
  or edx,GDPD_IS_LIST or GDPD_MEMORY_ONLY

  push pedx		     ;flags
  push psize		     ;len
  push peax		     ;data
  mov pedx,pdword [pesp+psize*3] ;pop the pointer to ControlPanelOption
  push pdword [pedx+psize+8] ;name
  push 0		     ;ClientSession
  push 0		     ;ListenSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call SavePluginData_asm

NoUserValueOption:
  pop peax    ;restore new ControlPanelOption

  lea pedx,pdword [pedi+6*psize]  ;pointer to subOptions

  ;Add ControlPanelOption to the linkedlist
  push peax ;ControlPanelOption structure
  push pedx ;linkedlist with ControlPanelOption structures
  call addToLinkedList

  mov eax,dword [pebx]
  test eax,eax
  jnz compairSizeSub

  mov eax,SIZEOF_CONTROLPANELOPTION

  compairSizeSub:

  ;substract the size of the man member
  sub ecx,eax
  ;get nextItem
  add pebx,peax

  jpcxz returnSetControlPanelOptions

  ;is this structure size greater than the size of the total strucure... return
  cmp ecx,dword [pebx]
  jae loopSubOptionsStructure


;mov eax,NO_ERROR
xor eax,eax
endSetControlPanelOptions:
call setLastError

returnSetControlPanelOptions:

call dleave
ret psize*2

;------------------------------------------------------------------------------
;This function will activate the protocol
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    AcceptProtocol_c:
    StartProc 1

    mov rcx,retAcceptProtocol_c
    xchg qword [rsp],rcx
    jmp AcceptProtocol_asm

    retAcceptProtocol_c:
    ;mov qword [rsp],rcx
    sub rsp,1*psize
    ;ret
    EndProc 1
    jmp rcx
else
    AcceptProtocol_c:
end if

;---------
;asm implementation
;---------
AcceptProtocol_asm:
call denter

mov pecx,endAcceptProtocol
call checkProtocolSession

mov pedx,peax ;safe internal ListenSession

;set status
mov dword [peax+7*psize],PT_RUNNING

mov pebx,pdword [pebp+psize*2]	 ;save psess

;Open key to "Protocols"
push 0		     ;No subkey
push 0		     ;No subkey
push 0		     ;No subkey
push subkeyProtocol  ;subKey
push pdword [hKey]   ;hKey
push 0		     ;Create key
call OpenRegKey
test pecx,pecx
jnz dontLoadOptionsFromRegistry

mov pdword [pebp+psize*2],peax	 ;set register key
mov peax,pedx ;restore internal ListenSession

;call getBindInitOptionFromRegistry to collect the init options of this protocol
mov pedi,pdword [peax+psize*2]	 ;get copyPsess
mov pedi,pdword [pedi+psize]	 ;get protocolName=
call getBindInitOptionFromRegistry

if used is32
  push peax    ;save pointer to BindInitOptions
end if

;close the key to "Protocols"
push pdword [pebp+psize*2]
call SafeRegCloseKeycall

if used is32
  pop peax
  test peax,peax
else
  test r12,r12
end if
jz dontLoadOptionsFromRegistry

if used is32
  push peax
  mov pedi,peax   ;(R/E)DI = pointer to first BindInitOptions
else
  push r12
  mov pedi,r12	  ;(R/E)DI = pointer to first BindInitOptions
end if

;call loopInitOptions to set the init options to the protocol inner structure
if used is32
  mov peax,pebx ;psess
  mov pdword [pebp+psize*2],peax ;restore psess
else
  mov r12,pebx	;pesess
  mov pdword [pebp+psize*2],r12 ;restore psess
end if
xor ebx,ebx ;clear internalListenSession value
mov ecx,dword [pedi] ;ammount of BindInitOption structures
  sub ecx,8				 ;substract member size and flags
  if used is32
    shr ecx,4				 ;ECX = ECX / 16 (16 bytes is one BindInitOption structure)
  else
    mov r14,rdx 			 ;save rdx

    ;divide by 24 (24 bytes is one BindInitOption structure)
    xor edx,edx
    mov eax,ecx
    mov ecx,24
    div ecx
    mov rcx,rax

    mov rdx,r14 			 ;restore rdx
  end if

add pedi,8	     ;seek to first BindInitOption
call loopInitOptions

pop pesi  ;restore BindInitOptions
push pesi ;push for FreeMemLocation call

call doDestructBindInitOptions

call FreeMemLocation ;free BindInitOptions

dontLoadOptionsFromRegistry:
if used is64
  fixStack 0	;align stack on 0x10  (win x64 calling convention)
  sub pesp,0x10 ;create shadow places for the function call (win x64 calling convention)
end if

;call the registerd ProtocolIsLoaded functions
push [registerdProtocolIsLoadedFunctions]
call CountItems
mov ecx,eax

jecxz endLoopProtocolIsLoadedCallbackFunctions ;jump when there are no callback functions registerd

loopProtocolIsLoadedCallbackFunctions:
  push pecx
  push [registerdProtocolIsLoadedFunctions]
  call GetDwordItem
  test peax,peax
  jz endLoopProtocolIsLoadedCallbackFunctions

  mov pesi,pecx ;save ecx

  argpush 1,pebx ;psess
  call peax ;call ProtocolIsLoaded Callback Function

  mov pecx,pesi ;restore ecx

nextLoopProtocolIsLoadedCallbackFunctions:
  loop loopProtocolIsLoadedCallbackFunctions
endLoopProtocolIsLoadedCallbackFunctions:

endAcceptProtocol:
call dleave
ret psize*1

;------------------------------------------------------------------------------
;This function will load the protocols of the given modules
;input:
;  1:Modulename
;  2:flags
;       NOT_SAVE_ON_DISK(0x1): the filename will not be stored on disk
;------------------------------------------------------------------------------
loadPlugin:
call denter
MakeStack

xor pecx,pecx ;clear register ecx/rcx

;load the module in memory
push pdword [pebp+(psize*2)]
call loadmodule
test peax,peax
jz endStart

mov pedx,peax

;check if the module is already loaded
push pedx
call getInternalModuleStructure
test peax,peax
jnz endLoadPlugin

jmp skipSleepSetLoadingModule

;for secure reasons, only load one module a time, this is a race for this right
setLoadingModule:
	push 1
	call sleep

	skipSleepSetLoadingModule:
	xor peax,peax ;Comparand
lock cmpxchg pdword [currentLoadingModule], pedx    ;Destination, Exchange,
jnz setLoadingModule


mov esi,dword [pedx+0x3C] ;get offset of PE header
cmp word [pedx+pesi+0x4+0x10],0x18  ;if SizeOfOptionalHeader<0x18 then BaseOfCode is not defined.
jb signatureNotFound

;mov eax,dword [peax+pedx+0x4+0x14+0x3C] ;[Base + signature + fileheader + offset of SizeOfHeaders in optional header] = SizeOfHeaders
mov esi,dword [pedx+pesi+0x4+0x14+0x14] ;[Base + offset PE header + signature + fileheader + offset of BaseOfCode in optional header] = BaseOfCode
test esi,esi
jz signatureNotFound
add pesi,pedx  ;pointer to first section with code

call pushServerSignature
db "TV's server 3.00"
pushServerSignature:

pop pedi

mov ecx,16/psize
if used is32
  repz cmpsd
else
  repz cmpsq
end if
jnz signatureNotFound

lodsp ;get pointer to __asm_exchange_tables
test peax,peax
jz endStart

push __asm_get_export_table_c
push __asm_get_export_table
call peax

jmp ReceivedExchangePointer

signatureNotFound:

;get pointer to function __asm_exchange_tables
push __asm_exchange_tables_name
push pedx
call getfunc
test peax,peax
jz endStart

mov pesi,peax
call denter
MakeStack

argpush 2,__asm_get_export_table_c
argpush 1,__asm_get_export_table
call pesi

call dleave


ReceivedExchangePointer:
push pedx
call getInternalModuleStructure
test peax,peax
jz endStart


;save the internamModuleStructure on non-violent register r12 or ecx
mov nvedx,peax

;mov pebx,[modules]
mov pebx,pdword [nvedx+(psize*(2+GetProtocolNames))]

;clear size on stack for the size paramter
if used is64
   lea rsi,qword [rsp+0x8*3]
   mov qword [rsi],0
else
   push 0
   mov esi,esp
end if

saveVolatileRegister

;call GetProtocolNames to retrieve the size of the protocol string
argpush 2,pesi ;size
argpush 1,0
call pebx ;GetProtocolNames
test peax,peax
jz endStart

;alloc memory for the protocol names
push pdword [pesi]
call GetMemLocation
mov pedi,peax

;get the names of the protocols supported by this plugin
argpush 2,pesi
argpush 1,pedi
call pebx ;GetProtocolNames

restoreVolatileRegister

loopThroughProtocols:
   mov peax,pdword [pebp+psize*2]

  push pdword [pebp+psize*2] ;module name
  push pedi		     ;protocol name
  call IsPluginToBlocked
  test eax,eax
  jnz loadNextProtocol

   mov peax,pdword [pebp+psize*2]

  ;Create ProtocolSession and internalProtocolSession
  push pedi
  call CreateProtocolSession
  mov pecx,peax

  push pedi
  call CreateProtocolSession


  ;Create internalProtocolStructure (with ipsess, psess and pointer to the module structure)
  push nvedx ;p module structure
  push peax  ;iPsess
  push pecx  ;psess
  call CreateInternalProtocolStructure


  push peax
  mov peax,pdword [peax]
  pop peax

  ;Add the structure to the list
  push peax
  push protocols
  call addToLinkedList

  saveVolatileRegister


  if used isWin
    if used is64
      fixStack 0 ;fixup stack
    end if
  end if

  if used isLinux
    if used is64
      push rdi	  ;save RDI
    end if
  end if

  ;call StartProtocol

  argpush 1,pecx ;psess
  call pdword [nvedx+(psize*(2+StartProtocol))]

  if used isLinux
    if used is64
      pop rdi	 ;restore RDI
    end if
  end if

  ;mov peax,pdword [peax+psize*1]

  ;--push tempTitel
  ;--push pdword [pecx+psize*1]
  ;--call message




  restoreVolatileRegister

loadNextProtocol:
  push pedi
  call strlen

  inc peax
  add pedi,peax

  cmp byte [pedi],0
jne loopThroughProtocols

test pdword [pebp+psize*3],0x1 ;NOT_SAFE_ON_DISK
jnz endStart

push peax ;safe return value

mov pesi,pdword [pebp+psize*2] ;module filename

;count string size
push pesi
call strlen

add pesi,peax ;(R/E)SI = pointer to end of filename string
mov pecx,peax ;ECX = size of filename string
mov pebx,peax ;EBX = backup of filename size
std ;set direction downwards

findFileName:
  lodsb
  cmp al,'\'
  je foundName
  cmp al,'/'
  je foundName
loop findFileName

dec pesi
dec pesi

foundName:
inc pesi
inc pesi
cld ;set direction upwards

push REG_SZ		    ;valueType
push pebx		    ;value
push pdword [pebp+psize*2]  ;valueData
push pesi		    ;valueName
push 0			    ;no subKey
push 0			    ;no subKey
push 0			    ;no subKey
push subkeyMudules	    ;subKey
push pdword [hKey]	    ;hkey
call WriteRegEx

pop peax ;restore return value

endStart:
;--push pecx
;--call unloadmodule

mov [currentLoadingModule],0

endLoadPlugin:
call dleave
ret (psize*2)

;------------------------------------------------------------------------------
;This function will send the export table of this program  to a assembly based plugin
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    __asm_get_export_table_c:
    StartProc 1

    mov rcx,ret__asm_get_export_table_c
    xchg qword [rsp],rcx

    call denter
    mov pesi,beginExportList_c ;(R/E)SE => pouinter to list with export functions to copy
    jmp __asm_get_export_table_jmp

    ret__asm_get_export_table_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    __asm_get_export_table_c:
end if

;---------
;asm implementation
;---------

__asm_get_export_table:
call denter

mov pesi,beginExportList ;(R/E)SE => pouinter to list with export functions to copy

__asm_get_export_table_jmp:
if used is64
   xor pecx,pecx
end if

mov pedi, pdword [pebp+(psize*2)] ;pointer to asm_base_export structre
mov ecx,dword [pedi]

;if size=>export_list_size then size=export_list_size
cmp ecx,export_list_size
jbe copy_export_list

mov ecx,export_list_size

copy_export_list:
add pedi,4 ;skip the size member to get pointer of first function to replace

;copy byte[esi]->byte[edi] and increment both registers ECX times
rep movsb

xor peax,peax
inc eax

call dleave
ret (psize*1)


;------------------------------------------------------------------------------
;This function will recieve the export table of the plugin that is loading by function loadPlugin
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    __asm_set_export_table_c:
    StartProc 1

    mov rcx,ret__asm_set_export_table_c
    xchg qword [rsp],rcx
    jmp __asm_set_export_table

    ret__asm_set_export_table_c:
    sub rsp,1*psize
    EndProc 1
    jmp rcx
else
    __asm_set_export_table_c:
end if

;---------
;asm implementation
;---------

__asm_set_export_table:
call denter

mov pesi, pdword [pebp+(psize*2)] ;The pointer to a  asm_plugin_exports structure
mov ecx, dword [pesi] ;move size member to register

;if size=>import_list_size then size=import_list_size
cmp ecx,import_list_size
jbe copy_import_list

mov ecx,import_list_size

copy_import_list:
;Create an module structure
push import_list_size + SIZEOF_MODULE;
call GetMemLocation

;fill in the moduleHandle
mov pedx,[currentLoadingModule]
mov pdword [peax+(psize*1)], pedx

mov pedx,peax ;move the register pointer module structure to edx/rdx

mov pedi,pedx
add pedi,(psize*2) ;the pointer to the import list

add pesi,4 ;skip the len member

;copy byte[esi]->byte[edi] and increment both registers ECX times
rep movsb

;fill in the member nextModule
push pedx
push modules
call addToLinkedList

xor peax,peax
inc eax

call dleave
ret (psize*1)